/*==========================================================================*\
|                                                                            |
|                                                                            |
| PC-side Bootstrap Loader communication Application                         |
|                                                                            |
|                                                                            |
|----------------------------------------------------------------------------|
| Developed using:      Microsoft Visual C++ Version 6.0                     |
|----------------------------------------------------------------------------|
| Author:               L.Westlund                                           |
*/
#define VERSION 2.01
/*
| Initial Version:      09-22-08                                             |
| Last Change:          01-16-15                                             |
|----------------------------------------------------------------------------|
| Version history:                                                           |
| 1.00 09/22/08 L.Westlund   Initial version, published as 5xxBSL_Demo       |
| 1.01 01/16/09 L.Westlund   Added Scripting Language and support for ROM BSL|
| 1.02 02/06/09 L.Westlund   Added support for USB / Major interface redesign|
| 1.04 03/09/10 L.Westlund   Removed USB Timeout error                       |
|      04/26/10 L.Westlund   USB/UART support for RX from BSL in USB/5438    |
| 1.05 05/31/10 L.Westlund   Invoke error fixed                              |
| 1.06 08/18/10 L.Westlund   Changed 5438 invoke command for clarity         |
| 1.07 02/19/13 L.Westlund   Added baud rate invoke flexibility              |
|                            Added support for COMs > 9                      |
| 2.00 01/16/15 R.Heilmaier  Added I2C and SPI                               |
|                            Added MSP432 support                            |
|----------------------------------------------------------------------------|
| Designed 2009 by Texas Instruments Germany                                 |
\*==========================================================================*/

#include "BSL_Interface.h"
#include "TextFileIO.h"
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <conio.h>


void setMode();
void openFile(unsigned char *fileName);
void errorExit(unsigned char *error);

FILE* scriptFile;
unsigned char *token;
unsigned char line[100];
unsigned int reply;
unsigned char verbose_mode = 0;
unsigned char family;

/*******************************************************************************
* MAIN
*******************************************************************************/
int main(unsigned int argc, unsigned char* argv[])
{
	SYSTEMTIME lt;
	printf("\n------------------------------------------\n");
	printf("BSL Scripting application %3.2f\n", VERSION);
	GetLocalTime(&lt);
	printf("The local time is: %02d:%02d on %02d.%02d.%04d\n", lt.wHour, lt.wMinute, lt.wDay, lt.wMonth, lt.wYear);
	printf("------------------------------------------\n");

	if (argc == 1)
	{
		printf("How to use BSL_Scripter: BSL_Scripter.exe <script_name.txt>\n\n");
		return -1;
	}
	openFile(argv[1]);
	while (fgets(line, sizeof(line), scriptFile))
	{
		if (line[strlen(line) - 1] == '\n')
		{
			line[strlen(line) - 1] = ' ';
		}
		if (line[strlen(line) - 2] == '\r')
		{
			line[strlen(line) - 2] = ' ';
		}
		if ((memcmp(line, "//", 2) == 0))
		{
			// skip comments
		}

/***MODE***********************************************************************/
		else if ((memcmp(line, "MODE", 4) == 0))
		{
			setMode();
		}
/***CHANGE_BAUD_RATE***********************************************************/
		else if (memcmp(line, "CHANGE_BAUD_RATE", 16) == 0)
		{
			unsigned int rate;
			sscanf(line, "%*s %u", &rate);
			printf("Changing Baud Rate to %u baud\t\t", rate);
			if (rate == 9601)
			{
				// do nothing, initial sequence
			}
			else
			{
				reply = BSL_changeBaudRate(rate);
				if (reply == BSL_ACK)
				{
					printf("DONE\n");
				}
				else
				{
					printf("FAIL(0x%2.2X)\n", reply);
				}
			}
		}
/***CRC_32*********************************************************************/
		else if (memcmp(line, "CRC_32", 6) == 0)
		{
			unsigned int addr, length, value, outputValue;
			if (sscanf(line, "%*s %x %x %x", &addr, &length, &value) == 3)
			{
				outputValue = 0;
				printf("CRC from 0x%X of %u bytes to 0x%4.4X\t\t", addr, length, value);
			}
			else
			{
				outputValue = 1;
				printf("CRC from 0x%X of %u bytes\t\t\t", addr, length);
			}
			reply = BSL_CRC_Check_32(addr, length);
			if (reply == BSL_ACK)
			{
				unsigned int crc;
				dataBuffer db = BSL_get_RX_Buffer();
				crc = db.data[1];
				crc |= db.data[2] << 8;
				if (outputValue)
				{
					printf("0x%4.4X\n", crc);
				}
				else if (crc == value)
				{
					printf("MATCH\n");
				}
				else if (crc != value)
				{
					printf("NOT A MATCH (0x%4.4X)\n", crc);
				}
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***CRC*********************************************************************/
		else if (memcmp(line, "CRC", 3) == 0)
		{
			unsigned int addr, length, value, outputValue;
			if (sscanf(line, "%*s %x %x %x", &addr, &length, &value) == 3)
			{
				outputValue = 0;
				printf("CRC from 0x%X of %u bytes to 0x%4.4X\t\t\t", addr, length, value);
			}
			else
			{
				outputValue = 1;
				printf("CRC from 0x%X of %u bytes\t\t\t", addr, length);
			}
			reply = BSL_CRC_Check(addr, length);
			if (reply == BSL_ACK)
			{
				unsigned int crc;
				dataBuffer db = BSL_get_RX_Buffer();
				crc = db.data[1];
				crc |= db.data[2] << 8;
				if (outputValue)
				{
					printf("0x%4.4X\n", crc);
				}
				else if (crc == value)
				{
					printf("MATCH\n");
				}
				else if (crc != value)
				{
					printf("NOT A MATCH (0x%4.4X)\n", crc);
				}
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***DELAY*************************************************************************/
		else if (memcmp(line, "DELAY", 5) == 0)
		{
			int del;
			sscanf(line, "%*s %d", &del);
			printf("Waiting %u milliseconds\t\t\t", del);
			Sleep(del);
			printf("DONE\n");
		}
/***ERASE_CHECK**************************************************************/
		/*BSLDEMO2.exe*/
		else if (memcmp(line, "ERASE_CHECK", 11) == 0)
		{
			unsigned long int length, addr;
			sscanf(line, "%*s %x %x", &addr, &length);
			printf("Checking %4.4u bytes starting at 0x%4.4X for erase:\t\t\t", length, addr);
			reply = BSL_eraseCheck(addr, length);
			if (reply == BSL_ACK)
			{
				printf("PASSED\n");
			}
			else
			{
				printf("FAIL\n");
			}
		}
/***ERASE_MAIN_OR_INFO************************************************************/
		/*BSLDEMO2.exe*/
		else if (memcmp(line, "ERASE_MAIN_OR_INFO", 18) == 0)
		{
			unsigned int addr;
			sscanf(line, "%*s %x", &addr);
			printf("Erasing main/info (addr:0x%X)\t\t\t", addr);
			reply = BSL_eraseMainOrInfo(addr);
			if (reply == BSL_ACK)
			{
				printf("DONE\n");
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***ERASE_SEGMENT_32***************************************************/
		else if (memcmp(line, "ERASE_SEGMENT_32", 16) == 0)
		{
			unsigned int addr;
			sscanf(line, "%*s %x", &addr);
			printf("ERASE SEGMENT 32:\n    Erasing segment at addr 0x%2.2X\t\t", addr);
			reply = BSL_eraseSegment_32(addr);
			if (reply == BSL_ACK)
			{
				printf("DONE\n");
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***ERASE_SEGMENT***************************************************/
		else if (memcmp(line, "ERASE_SEGMENT", 13) == 0)
		{
			unsigned int addr;
			sscanf(line, "%*s %x", &addr);
			printf("ERASE SEGMENT:\n    Erasing segment at addr 0x%2.2X\t\t", addr);
			reply = BSL_eraseSegment(addr);
			if (reply == BSL_ACK)
			{
				printf("DONE\n");
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***LOAD_PC_32 ***********************************************************************/
		else if (memcmp(line, "SET_PC_32", 9) == 0)
		{
			unsigned int addr;
			sscanf(line, "%*s %x", &addr);
			printf("LOAD PC 32:\n    LOADING PC to 0x%X:\t\t\t", addr);
			BSL_LoadPC_32(addr);
			printf("SENT\n");
		}
/***LOAD_PC ***********************************************************************/
		else if (memcmp(line, "SET_PC", 6) == 0)
		{
			unsigned int addr;
			sscanf(line, "%*s %x", &addr);
			printf("LOAD PC:\n    LOADING PC to 0x%X:\t\t\t", addr);
			BSL_LoadPC(addr);
			printf("SENT\n");
		}
/***MASS_ERASE*******************************************************************/
		else if (memcmp(line, "MASS_ERASE", 10) == 0)
		{
			printf("Mass Erase:\t\t\t\t\t");
			reply = BSL_massErase(family);
			if (reply == BSL_ACK)
			{
				if (family == FAMILY_FRAM)
				{
					printf("SENT\n");
				}
				else
				{
					printf("DONE\n");
				}
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***REBOOT_RESET*****************************************************************/
		else if (memcmp(line, "REBOOT_RESET", 12) == 0)
		{
			printf("Triggering reboot reset\t\t\t\t");
			BSL_rebootReset();
			printf("SENT\n");
		}
/***RX_DATA_BLOCK_FAST**********************************************************/
		else if (memcmp(line, "RX_DATA_BLOCK_FAST", 17) == 0)
		{
			char filename[50];
			sscanf(line, "%*s %s", filename);
			printf("RX DATA BLOCK FAST:\n");
			printf("    Writing %s to device (fast):\n", filename);
			reply = BSL_RX_TXT_File(filename, 1, HALFWORD_ADDRESSED);
			printf("    SENT\n");
		}
/***RX_DATA_BLOCK_32***********************************************/
		else if (memcmp(line, "RX_DATA_BLOCK_32", 16) == 0)
		{
			char filename[50];
			sscanf(line, "%*s %s", filename);
			printf("RX DATA BLOCK 32:\n    Writing %s to device:\n", filename);
			reply = BSL_RX_TXT_File(filename, 0, WORD_ADDRESSED);
			if (reply == BSL_ACK)
			{
				printf("    DONE\n");
			}
			else
			{
				printf("    FAIL(0x%2.2X)\n", reply);
			}
		}
/***RX_DATA_BLOCK***********************************************/
		else if (memcmp(line, "RX_DATA_BLOCK", 13) == 0)
		{
			char filename[50];
			sscanf(line, "%*s %s", filename);
			printf("RX DATA BLOCK:\n    Writing %s to device:\n", filename);
			reply = BSL_RX_TXT_File(filename, 0, HALFWORD_ADDRESSED);
			if (reply == BSL_ACK)
			{
				printf("    DONE\n");
			}
			else
			{
				printf("    FAIL(0x%2.2X)\n", reply);
			}
		}
/***RX_PASSWORD_32*************************************************************/
		else if (memcmp(line, "RX_PASSWORD_32", 14) == 0)
		{
			char filename[100];
			DataBlock pass = default_pass;
			printf("RX Password 32:\t\t\t\t\t");
			if (sscanf(line, "%*s %s", filename) == 1)
			{
				if (openTI_TextForRead(filename) == 0)
				{
					pass = readTI_TextFile(256);
					reply = BSL_RX_Password_32(pass);
					if (reply == BSL_ACK)
					{
						printf("DONE\n");
					}
					else
					{
						printf("FAIL(0x%2.2X)\n", reply);
					}
				}
				else
				{
					printf("\n    FAIL cannot open %s\n", filename);
				}
			}
			else
			{
				reply = BSL_RX_Password_32(pass);
				if (reply == BSL_ACK)
				{
					printf("DONE\n");
				}
				else
				{
					printf("FAIL(0x%2.2X)\n", reply);
				}
			}
		}
/***RX_PASSWORD*************************************************************/
		else if (memcmp(line, "RX_PASSWORD", 11) == 0)
		{
			char filename[50];
			DataBlock pass = default_pass;
			printf("RX Password:\t\t\t\t\t");
			if (sscanf(line, "%*s %s", filename) == 1)
			{
				openTI_TextForRead(filename);
				pass = readTI_TextFile(32);
			}
			reply = BSL_RX_Password(pass);
			if (reply == BSL_ACK)
			{
				printf("DONE\n");
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***SET_MEMORY_OFFSET**********************************************************/
		/*BSLDEMO2.exe*/
		else if (memcmp(line, "SET_MEMORY_OFFSET", 17) == 0)
		{
			unsigned int addr;
			sscanf(line, "%*s %x", &addr);
			printf("Setting offset to 0x%X\t\t\t", addr);
			reply = BSL_setMemOffset(addr);
			if (reply == BSL_ACK)
			{
				printf("DONE\n");
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}
/***TOGGLE_INFO****************************************************************/
		else if (memcmp(line, "TOGGLE_INFO", 11) == 0)
		{
			printf("Toggling info flag\t\t\t\t");
			reply = BSL_toggleInfo();
			if (reply == BSL_ACK)
			{
				printf("DONE\n");
			}
			else
			{
				printf("FAIL(0x%2.2X)\n", reply);
			}
		}

/***TX_DATA_BLOCK_32**********************************************/
		else if (memcmp(line, "TX_DATA_BLOCK_32", 16) == 0)
		{
			unsigned int length, addr;
			char filename[50];
			sscanf(line, "%*s %x %x %s", &addr, &length, filename);
			printf("TX DATA BLOCK 32:\n    Reading %4.4u bytes starting at 0x%4.4X to file %s:\n", length, addr, filename);
			reply = BSL_TX_TXT_File(filename, addr, length, WORD_ADDRESSED);
			if (reply == BSL_ACK)
			{
				printf("    DONE\n");
			}
			else
			{
				printf("    FAIL(0x%2.2X)\n", reply);
			}
		}
/***TX_DATA_BLOCK**********************************************/
		else if (memcmp(line, "TX_DATA_BLOCK", 13) == 0)
		{
			unsigned int length, addr;
			char filename[50] = "B.txt";
			sscanf(line, "%*s %x %x %s", &addr, &length, filename);
			printf("TX DATA BLOCK:\n    Reading %4.4u bytes starting at 0x%4.4X to file %s:\n", length, addr, filename);
			reply = BSL_TX_TXT_File(filename, addr, length, HALFWORD_ADDRESSED);
			if (reply == BSL_ACK)
			{
				printf("    DONE\n");
			}
			else
			{
				printf("    FAIL(0x%2.2X)\n", reply);
			}
		}
/***BSL_VERSION_32****************************************************************/
		else if (memcmp(line, "TX_BSL_VERSION_32", 17) == 0)
		{
			printf("BSL Version 32:\n    %s\n", BSL_TX_BSL_Version_String_32());
		}
/***BSL_VERSION****************************************************************/
		else if (memcmp(line, "TX_BSL_VERSION", 14) == 0)
		{
			printf("BSL Version:\n    %s\n", BSL_TX_BSL_Version_String());
		}
/***TX_BUFFER_SIZE*************************************************************/
		else if (memcmp(line, "TX_BUFFER_SIZE", 14) == 0)
		{
			signed int bufSize = BSL_TX_BufferSize();
			printf("Getting Buffer Size:\t\t\t\t");
			if (bufSize > 0)
			{
				printf("[%u]\n", bufSize);
			}
			else
			{
				printf("FAIL(0x%X)\n", bufSize);
			}
		}
/***VERBOSE***********************************************************************/
		else if (memcmp(line, "VERBOSE", 7) == 0)
		{
			printf("Verbose mode ");
			verbose_mode ^= 0x01;
			BSL_SetVerbose(verbose_mode);
			if (verbose_mode)
			{
				printf("on \n");
			}
			else
			{
				printf("off \n");
			}
		}
		else
		{
			printf("Unknown command\n");
		}
	}
	return 0;
}

/*******************************************************************************
*Function:     setMode
*Description:  This function sets the mode that will be used
*Parameters:
*              none
*Returns:
*              none
*******************************************************************************/
void setMode()
{
	token = strtok(line, " ");  // get command
	token = strtok(NULL, " ");  // get family
	printf("Initializing, Mode: ");
	if (memcmp(token, "543x_family", 11) == 0)
	{
		BSL_setFamily(FAMILY_5438);
		family = FAMILY_5438;
		printf("5438  ");
	}
	else if (memcmp(token, "ROM", 3) == 0)
	{
		BSL_setFamily(FAMILY_ROM);
		family = FAMILY_ROM;
		printf("ROM   ");
	}
	else if (memcmp(token, "5xx", 3) == 0)
	{
		BSL_setFamily(FAMILY_FLASH);
		family = FAMILY_FLASH;
		printf("5xx ");
	}
	else if (memcmp(token, "6xx", 3) == 0)
	{
		BSL_setFamily(FAMILY_FLASH);
		family = FAMILY_FLASH;
		printf("6xx ");
	}
	else if (memcmp(token, "FRxx", 4) == 0)
	{
		BSL_setFamily(FAMILY_FRAM);
		family = FAMILY_FRAM;
		printf("FRxx ");
	}
	else if (memcmp(token, "P4xx", 4) == 0)
	{
		BSL_setFamily(FAMILY_MSP432);
		family = FAMILY_MSP432;
		printf("P4xx ");
	}
	else
	{
		errorExit("Error parsing MODE");
	}
	token = strtok(NULL, " ");  // get COM or protocol
	long rate = 0;
	if (memcmp(token, "COM", 3) == 0)
	{
		BSL_setUpConnection(token, UART, UART_STANDARD_BAUDRATE, family); // 9600 baud UART
	}
	else if (memcmp(token, "USB", 3) == 0)
	{
		BSL_setCom(USB);
		printf("USB: %s", token);
		BSL_initialize_BSL("", 0, 0, family);
	}
	else if (memcmp(token, "I2C", 3) == 0)
	{
		// I2C
		token = strtok(NULL, " ");  // get baudrate
		if (memcmp(token, "COM", 3) == 0)
		{
			BSL_setUpConnection(token, I2C, I2C_STANDARD_BAUDRATE, family); // 100 kHz I2C
		}
		else
		{
			rate = strtol(token, NULL, 10);
			token = strtok(NULL, " ");
			switch (rate)
			{
				case 100000: // 100kHz
					BSL_setUpConnection(token, I2C, rate, family);
					break;
				case 100001: // 100.001 Hz
					BSL_setUpConnection(token, I2C, rate, family);
					break;
				case 400000: // 400kHz
					BSL_setUpConnection(token, I2C, rate, family);
					break;
				case 400001: // 400.001 Hz
					BSL_setUpConnection(token, I2C, rate, family);
					break;
				default:
					errorExit("Error parsing I2C");
					break;
			}
		}
	}
	else if (memcmp(token, "SPI", 3) == 0)
	{
		token = strtok(NULL, " ");  // get baudrate
		if (memcmp(token, "COM", 3) == 0)
		{
			BSL_setUpConnection(token, SPI, SPI_STANDARD_BAUDRATE, family); // 125kHz SPI
		}
		else
		{
			rate = strtol(token, NULL, 10);
			token = strtok(NULL, " ");
			switch (rate)
			{
				case 125000: // 125kHz
					BSL_setUpConnection(token, SPI, rate, family);
					break;
				case 125001: // 125001 Hz
					BSL_setUpConnection(token, SPI, rate, family);
					break;
				case 250000: // 250kHz
					BSL_setUpConnection(token, SPI, rate, family);
					break;
				case 1000000: // 1MHz
					BSL_setUpConnection(token, SPI, rate, family);
					break;
				case 1000001: // 1.000.001 Hz
					BSL_setUpConnection(token, SPI, rate, family);
					break;
				default: 
					errorExit("Error parsing SPI");
					break;
			}		
		}
	}
	else if (memcmp(token, "UART", 4) == 0)
	{
		token = strtok(NULL, " ");  // get baudrate
		if (memcmp(token, "COM", 3) == 0)
		{
			BSL_setUpConnection(token, UART, UART_STANDARD_BAUDRATE, family); // 9600 baud UART
		}
		else
		{
			rate = strtol(token, NULL, 10);
			token = strtok(NULL, " ");
			switch (rate)
			{
				case 1200: // 1200 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 2400: // 2400 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 4800: // 4800 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 4801: // 4801 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 4802: // 4802 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 9600: // 9600 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 9601: // 9601 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 19200: // 19200 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 38400: // 38400 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 57600: // 57600 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 115200: // 115200 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 230400: // 230400 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 460800: // 460800 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				case 921600: // 921600 baud
					BSL_setUpConnection(token, UART, rate, family);
					break;
				default:
					errorExit("Error parsing UART");
					break;
			}
		}
	}
	else
	{
		errorExit("Error parsing COM");
	}
	printf("\n");
}

/*******************************************************************************
*Function:     openFile
*Description:  This function opens a file for binary read
*Parameters:
*              unsigned char * fileName
*Returns:
*              none
*******************************************************************************/
void openFile(unsigned char *fileName)
{
	scriptFile = fopen(fileName, "rb");
}

/*******************************************************************************
*Function:     errorExit
*Description:  This function prints an error message
*Parameters:
*              unsigned char * error
*Returns:
*              none
*******************************************************************************/
void errorExit(unsigned char *error)
{
	printf("\n-------------ERROR-------------\n%s\n-------------ERROR-------------\n", error);
}